@@ -2,9 +2,10 @@ import glob
 import html
 import os
 import re
+import sys
 import textwrap
 from pathlib import Path
-from typing import List, Union, Tuple, Optional, Any, Dict
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import pycountry
 import requests
@@ -18,8 +19,6 @@ class NFO:
     AUDIO_CHANNEL_LAYOUT_WEIGHT = {
         "LFE": 0.1
     }
-    SEASON_T = Optional[Union[int, str]]
-    EPISODE_T = Optional[Union[Tuple[Optional[int], Optional[str]]]]
     IMDB_ID_T = re.compile(r"^tt\d{7,8}$")
     TMDB_ID_T = re.compile(r"^(tv|movie)/\d+$")
     TVDB_ID_T = re.compile(r"^\d+$")
@@ -88,10 +87,7 @@ class NFO:
 
         return template
 
-    def set_config(self, file: str, season: SEASON_T, episode: EPISODE_T, **config: Any) -> None:
-        if not config or not isinstance(config, dict):
-            raise ValueError("NFO.set_config: Parameter config is empty or not a dictionary...")
-
+    def set_config(self, file: str, **config: Any) -> None:
         self.file = file
         self.media_info = MediaInfo.parse(self.file)
 
@@ -100,8 +96,8 @@ class NFO:
         self.note = config.get("note")
         self.preview = config.get("preview")
 
-        self.season = season
-        self.episode, self.episode_name = episode or (None, None)
+        self.season = config.get("season")
+        self.episode, self.episode_name = config.get("episode") or (None, None)
         self.episodes = self.get_tv_episodes()
         self.release_name = self.get_release_name()
 
@@ -109,6 +105,20 @@ class NFO:
         self.audio = self.media_info.audio_tracks
         self.subtitles = self.media_info.text_tracks
 
+        tracks_without_language = [
+            x for x in self.videos + self.audio + self.subtitles
+            if not x.language or x.language == "und"
+        ]
+        if tracks_without_language:
+            print("The following tracks have no language tag! All tracks need a language tag!")
+            for track in tracks_without_language:
+                print(f"{track.track_type} Track #{track.track_id} ({track.format}, {track.bit_rate / 1000} kb/s)")
+            print(
+                "Yes, even Video Track's have language e.g., Credits, Signs, Letters, Different Intro Sequence, etc.\n"
+                "Don't forget to verify and add language tags to the rest of the files too!"
+            )
+            sys.exit(1)
+
         chapters = next(iter(self.media_info.menu_tracks), None)
         if chapters:
             self.chapters = {
@@ -124,63 +134,66 @@ class NFO:
             self.chapters = {}
             self.chapters_numbered = False
 
-        self.imdb = self.get_imdb_id(config)
-        self.tmdb = self.get_tmdb_id(config)
-        self.tvdb = self.get_tvdb_id(config)
+        self.imdb = self.get_imdb_id(config.get("imdb"))
+        self.tmdb = self.get_tmdb_id(config.get("tmdb"))
+        self.tvdb = self.get_tvdb_id(config.get("tvdb"))
 
         self.title_name, self.title_year = self.get_title_name_year()
-        self.banner_image = self.get_banner_image(self.tvdb) if self.tvdb else None
+        self.banner_image = self.get_banner_image(self.tvdb) if self.tvdb and self.fanart_api_key else None
         self.preview_images = self.get_preview_images(self.preview) if self.preview else []
 
-        print(self)
-
-    def get_imdb_id(self, config: dict) -> str:
+    def get_imdb_id(self, imdb_id: Any) -> str:
         """
         Get an IMDB ID from either the media's global tags, or the config.
         Since IMDB IDs are required for this project, it will bug the user for
         one interactively if not found.
         """
-        general_track = self.media_info.general_tracks[0].to_data()
-        imdb_id = general_track.get("imdb") or config.get("imdb")
+        if not imdb_id:
+            general_track = self.media_info.general_tracks[0].to_data()
+            imdb_id = general_track.get("imdb")
         if not imdb_id:
             print("No IMDB ID was provided but is required...")
-        while not imdb_id:
+        while not imdb_id or not isinstance(imdb_id, str):
             user_id = input("IMDB ID (e.g., 'tt0487831'): ")
             if not self.IMDB_ID_T.match(user_id):
-                print(f"The provided IMDB ID '{user_id}' is not valid...")
+                print(f"The provided IMDB ID {user_id!r} is not valid...")
                 print("Expected e.g., 'tt0487831', 'tt10810424', (include the 'tt').")
             else:
                 imdb_id = user_id
         return imdb_id
 
-    def get_tmdb_id(self, config: dict) -> Optional[str]:
+    def get_tmdb_id(self, tmdb_id: Any) -> Optional[str]:
         """
         Get a TMDB ID from either the media's global tags, or the config.
         It will raise a ValueError if the provided ID is invalid.
         """
-        general_track = self.media_info.general_tracks[0].to_data()
-        tmdb_id = general_track.get("tmdb") or config.get("tmdb") or None
+        if not tmdb_id:
+            general_track = self.media_info.general_tracks[0].to_data()
+            tmdb_id = general_track.get("tmdb")
         if not tmdb_id:
             print("Warning: No TMDB ID was provided...")
             return None
-        if not self.TMDB_ID_T.match(tmdb_id):
-            print(f"The provided TMDB ID '{tmdb_id}' is not valid...")
+        if not self.TMDB_ID_T.match(tmdb_id) or not isinstance(tmdb_id, str):
+            print(f"The provided TMDB ID {tmdb_id!r} is not valid...")
             print("Expected e.g., 'tv/2490', 'movie/14836', (include the 'tv/' or 'movie/').")
             raise ValueError("Invalid TMDB ID")
         return tmdb_id
 
-    def get_tvdb_id(self, config: dict) -> Optional[int]:
+    def get_tvdb_id(self, tvdb_id: Any) -> Optional[int]:
         """
         Get a TVDB ID from either the media's global tags, or the config.
         It will raise a ValueError if the provided ID is invalid.
         """
-        general_track = self.media_info.general_tracks[0].to_data()
-        tvdb_id = general_track.get("tvdb") or config.get("tvdb") or None
+        if not tvdb_id:
+            general_track = self.media_info.general_tracks[0].to_data()
+            tvdb_id = general_track.get("tvdb")
         if not tvdb_id:
             print("Warning: No TVDB ID was provided...")
             return None
-        if not self.TVDB_ID_T.match(str(tvdb_id)):
-            print(f"The provided TVDB ID '{tvdb_id}' is not valid...")
+        if isinstance(tvdb_id, int):
+            tvdb_id = str(tvdb_id)
+        if not self.TVDB_ID_T.match(tvdb_id) or not isinstance(tvdb_id, str):
+            print(f"The provided TVDB ID {tvdb_id!r} is not valid...")
             print("Expected e.g., '79216', '1395', (not the url slug e.g., 'the-office-us').")
             raise ValueError("Invalid TVDB ID")
         return int(tvdb_id)
